package cn.fw.hermes.controller.app;

import static cn.fw.common.businessvalidator.Validator.BV;
import static cn.fw.common.web.util.ExceptionHandler.handleException;
import static cn.fw.common.web.util.ResultBuilder.failureWithMessage;
import static cn.fw.common.web.util.ResultBuilder.success;
import static cn.fw.hermes.common.constant.MessageStr.QUERY_FAILURE;
import static cn.fw.hermes.common.constant.MessageStr.SAVE_FAILURE;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.data.base.domain.common.Message;
import cn.fw.hermes.domain.enums.UserTypeEnum;
import cn.fw.hermes.domain.vo.AccountResultVO;
import cn.fw.hermes.service.biz.AccountBizService;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.enums.AuthType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @Author: Chenery
 * @Date: 2021/2/2 11:11
 */
@RestController
@Slf4j
@RequestMapping("/app/account")
@RequiredArgsConstructor
@Authorization(AuthType.APP)
public class AppAccountController {

    private final AccountBizService accountBizService;

    /**
     * 获取业务人员用户签名 userId 用户ID
     *
     * @return 签名信息
     */
    @GetMapping("/getSig")
    @IgnoreAuth
    public Message<AccountResultVO> getSig(@CurrentUser LoginAuthBean user) {
        final String msg = "获取用户签名[/app/account/getSig]";
        try {
            BV.isTrue(!ObjectUtils.isEmpty(user) && !ObjectUtils.isEmpty(user.getUserId()),
                    "loginUser不存在或者userId为空");
            Long userId = user.getUserId();
            log.info("{} : params[{}]", msg, userId);
            AccountResultVO sig = accountBizService.getLoginSig(userId, true);
            return success(sig, d -> log.info("{} 成功 : result[{}]", msg, d));
        } catch (Exception ex) {
            handleException(ex, e -> log.error("{} 失败: result:[{}]", msg, e));
            return failureWithMessage(SAVE_FAILURE);
        }

    }


    /**
     * 获取用户IM识别码
     * @param memberId 会员ID
     * @param userType 用户类型
     * @return 用户IM识别码
     */
    @GetMapping("/getIdentifier")
    @IgnoreAuth
    public Message<String> getIdentifier(Long memberId, Integer userType) {
        final String msg = "获取IM识别码[/app/account/getIdentifier]";
        try {
            log.info("{} : params[{},{}]", msg, memberId, userType);
            UserTypeEnum userTypeEnum = UserTypeEnum.ofValue(userType);
            BV.isTrue(!ObjectUtils.isEmpty(userTypeEnum),"用户类型不存在");
            String identifierCode = accountBizService
                    .queryIdentifierCode(memberId, userTypeEnum.getValue().equals(UserTypeEnum.B.getValue()) ? true : false);
            log.info("{} 成功：result[{}]",msg,identifierCode);
            return success(identifierCode);
        } catch (Exception e) {
            handleException(e, ex -> log.error("{} 失败，result[{}]", msg, e));
            return failureWithMessage(QUERY_FAILURE);
        }
    }


}
