package cn.fw.hermes.service.biz;

import static cn.fw.common.businessvalidator.Validator.BV;

import cn.fw.hermes.common.constant.CloudDataEnum;
import cn.fw.hermes.common.constant.PullChatApiCodeEnum;
import cn.fw.hermes.common.utils.DateUtils;
import cn.fw.hermes.common.utils.HttpClientUtils;
import cn.fw.hermes.common.utils.im.RedisKeyUtil;
import cn.fw.hermes.common.utils.im.TXCloudHelper;
import cn.fw.hermes.domain.db.Account;
import cn.fw.hermes.domain.db.ChatPullRecord;
import cn.fw.hermes.domain.db.ChatRecord;
import cn.fw.hermes.domain.dto.AdminConfigDto;
import cn.fw.hermes.domain.dto.DownResultDto;
import cn.fw.hermes.domain.dto.DownResultDto.File;
import cn.fw.hermes.domain.dto.ImChatMsgDto;
import cn.fw.hermes.domain.enums.UserTypeEnum;
import cn.fw.hermes.sdk.api.para.ChatPullCondition;
import cn.fw.hermes.sdk.api.para.ChatRecordCondition;
import cn.fw.hermes.sdk.api.result.ChatUserDto;
import cn.fw.hermes.sdk.api.result.ChatRecordResult;
import cn.fw.hermes.service.data.AccountService;
import cn.fw.hermes.service.data.ChatPullRecordService;
import cn.fw.hermes.service.data.ChatRecordService;
import com.alibaba.fastjson.JSON;
import java.io.IOException;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Optional;
import java.util.Scanner;
import java.util.zip.GZIPInputStream;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.lang.NonNull;
import org.springframework.stereotype.Service;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;

/**
 * @Author: Chenery
 * @Date: 2021/2/21 8:38
 */
@Service
@RequiredArgsConstructor
@Slf4j
public class ChatRecordBizService {

    private final ChatRecordService chatRecordService;
    private final ChatPullRecordService chatPullRecordService;
    private final AdminConfigDto adminConfigDto;
    private final AccountBizService accountBizService;
    private final AccountService accountService;

    /**
     * 拉取某年月日小时内的聊天记录
     *
     * @param timePoint 查询时间线
     */
    public void pullChatRecord(String timePoint) {
        String adminKey = RedisKeyUtil.getAdminkey(adminConfigDto.getAdminId());
        String adminSig = accountBizService
                .getSig(adminConfigDto.getAdminId(), 3600 * 24 * 10, adminKey).getSig();
        String url = TXCloudHelper
                .getUrl(CloudDataEnum.GET_HISTORY.getCode(), adminSig, adminConfigDto.getAdminId(),
                        adminConfigDto.getAppId());
        Map<String, String> params = new HashMap<>();
        params.put("ChatType", "C2C");
        params.put("MsgTime", timePoint);
        String result = HttpClientUtils.doPostJson(url, JSON.toJSONString(params));
        DownResultDto downResultDto = JSON.parseObject(result, DownResultDto.class);
        PullChatApiCodeEnum apiCodeEnum = PullChatApiCodeEnum.getEnum(downResultDto.getErrorCode());
        ChatPullRecord chatPullRecord = ChatPullRecord
                .create(timePoint, null, downResultDto.getErrorCode()
                        , apiCodeEnum.getMsg());
        chatPullRecordService.insert(chatPullRecord);
        if (downResultDto.getErrorCode().equals(PullChatApiCodeEnum.A.getCode()) &&
                "OK".equals(downResultDto.getActionStatus())) {
            List<File> files = downResultDto.getFile();
            files.forEach(this::accept);
        }


    }

    /**
     * 查询某时间段内某些用户聊天次数实现 筛选方式： 双方点对点的一次聊天符合条件，仅为单点单发消息不成立
     *
     * @param chatCount 聊天次数
     * @param userTypes 用户类型
     * @param startTime 开始时间
     * @param endTime   结束时间
     * @return 查询结果
     */
    public List<ChatUserDto> queryNumberOfChat(Integer chatCount, List<Integer> userTypes,
            Date startTime, Date endTime) {
        List<ChatUserDto> chatUserDtoList = chatRecordService
                .queryNumberOfChat(chatCount, userTypes, startTime, endTime);
        List<ChatUserDto> result = new ArrayList<>();
        if (!CollectionUtils.isEmpty(chatUserDtoList)) {
            List<ChatUserDto> copyList = new ArrayList<>(chatUserDtoList);
            for (ChatUserDto chatUserDto : chatUserDtoList) {
                for (ChatUserDto copyChatUserDto : copyList) {
                    if (chatUserDto.getFromUserId().equals(copyChatUserDto.getToUserId()) &&
                            chatUserDto.getFromUserType().equals(copyChatUserDto.getToUserType()) &&
                            chatUserDto.getToUserId().equals(copyChatUserDto.getFromUserId()) &&
                            chatUserDto.getToUserType().equals(copyChatUserDto.getFromUserType())) {
                        result.add(chatUserDto);
                        copyList.remove(copyChatUserDto);
                        copyList.remove(chatUserDto);
                        break;
                    }
                }
            }
        }
        return result;
    }


    /**
     * 查询在当前日期的上一天中业务人员和客户聊天次数大于chatCount的记录
     *
     * @param chatCount 聊天次数
     * @return 查询结果
     */
    public List<ChatUserDto> queryNumberOfChat(Integer chatCount) {
        List<Integer> userTypes = new ArrayList<>(Arrays.asList(UserTypeEnum.C.getValue(),
                UserTypeEnum.B.getValue()));
        Date yesterdayStartTime = DateUtils.dayBegin(DateUtils.addSomeDay(new Date(), -1));
        Date yesterdayEndTime = DateUtils.dayEnd(DateUtils.addSomeDay(new Date(), -1));
        return this.queryNumberOfChat(chatCount, userTypes, yesterdayStartTime, yesterdayEndTime);
    }


    /**
     * 拉取聊天记录
     *
     * @param file 聊天文件
     */
    private void accept(File file) {
        GZIPInputStream in = null;
        Scanner scanner = null;
        try {
            in = new GZIPInputStream(
                    new URL(file.getURL()).openConnection().getInputStream());
            scanner = new Scanner(in, StandardCharsets.UTF_8.name());
            ArrayList<ChatRecord> chatRecords = new ArrayList<>();
            while (scanner.hasNextLine()) {
                String line = scanner.nextLine();
                if (line.contains("MsgBody")) {
                    line = line.endsWith("}}]},") ? line.substring(0, line.lastIndexOf(","))
                            : line;
                    ImChatMsgDto imChatMsgDto = JSON.parseObject(line, ImChatMsgDto.class);
                    ChatRecord chatRecord = ChatRecord
                            .create(imChatMsgDto.getFrom_Account(),
                                    imChatMsgDto.getTo_Account(),
                                    new Date(imChatMsgDto.getMsgTimestamp() * 1000), line);
                    chatRecords.add(chatRecord);
                }
            }

            chatRecordService.insertBatch(chatRecords);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (in != null) {
                try {
                    in.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (scanner != null) {
                scanner.close();
            }
        }

    }

    /**
     * 查询指定时间范围内指定两人聊天记录 默认拉取给定时间范围内全部消息记录 应答中的"Complete": 1表示该时间范围的消息已全部拉取。 若续拉的应答里 Complete
     * 值为0，表明还需要继续续拉，直到 Complete 值为1。
     *
     * @param params  URL请求体
     * @param msgList 待添加的消息列表容器
     * @return 聊天记录集合
     */
    private void queryChatRecordOfCouple(@NonNull Map<String, Object> params,
            List<ChatRecordCondition> msgList) {
        //获取管理员签名
        String adminKey = RedisKeyUtil.getAdminkey(adminConfigDto.getAdminId());
        String adminSig = accountBizService
                .getSig(adminConfigDto.getAdminId(), 3600 * 24 * 10, adminKey).getSig();
        String url = TXCloudHelper
                .getUrl(CloudDataEnum.QUERY_SINGLE_CHAT.getCode(), adminSig,
                        adminConfigDto.getAdminId(),
                        adminConfigDto.getAppId());
        //调用IM接口发送请求
        String result = HttpClientUtils.doPostJson(url, JSON.toJSONString(params));
        ChatRecordResult chatRecordResult = JSON.parseObject(result, ChatRecordResult.class);
        BV.isTrue("OK".equals(chatRecordResult.getActionStatus()) && chatRecordResult.getErrorCode()
                .equals(0), "调用IM查询单聊消息接口失败 错误信息：" + chatRecordResult.getErrorInfo());
        if (!ObjectUtils.isEmpty(chatRecordResult.getMsgList())
                && chatRecordResult.getMsgList().size() > 0) {
            msgList.addAll(chatRecordResult.getMsgList());
        }
        //如果未拉取完整需要续拉
        if (chatRecordResult.getComplete().equals(0)) {
            params.put("MaxTime", chatRecordResult.getLastMsgTime());
            params.put("LastMsgKey", chatRecordResult.getLastMsgKey());
            this.queryChatRecordOfCouple(params, msgList);
        }

    }


    /**
     * 查询指定时间范围内指定两人聊天记录次数
     *
     * @param chatPullCondition 查询条件对象
     * @return 筛选后的聊天集合
     */
    public Integer queryChatRecordOfCouple(
            @NonNull ChatPullCondition chatPullCondition) {
        List<ChatRecordCondition> msgList = new ArrayList<>();
        //封装url请求体
        Optional<Account> from = accountService
                .queryAccountByUserId(chatPullCondition.getFromAccount(),
                        chatPullCondition.getFromIsStaff() ? UserTypeEnum.B.getValue()
                                : UserTypeEnum.C.getValue());
        BV.isTrue(from.isPresent(), "消息发送人不存在");
        Optional<Account> to = accountService
                .queryAccountByUserId(chatPullCondition.getToAccount(),
                        chatPullCondition.getToIsStaff() ? UserTypeEnum.B.getValue()
                                : UserTypeEnum.C.getValue());
        BV.isTrue(to.isPresent(), "消息接收人不存在");
        Map<String, Object> params = new HashMap<>();
        params.put("From_Account", chatPullCondition.getFromAccount());
        params.put("To_Account", chatPullCondition.getToAccount());
        params.put("MaxCnt", 100);
        params.put("MinTime", chatPullCondition.getStartTime());
        params.put("MaxTime", chatPullCondition.getEndTime());
        this.queryChatRecordOfCouple(params, msgList);
        List<ChatRecordCondition> result = new ArrayList<>();
        if (!CollectionUtils.isEmpty(msgList)) {
            List<ChatRecordCondition> copyList = new ArrayList<>(msgList);
            for (ChatRecordCondition chatRecordCondition : msgList) {
                for (ChatRecordCondition copyChatUserDto : copyList) {
                    if (chatRecordCondition.getFromAccount().equals(copyChatUserDto.getToAccount())
                            &&
                            chatRecordCondition.getToAccount()
                                    .equals(copyChatUserDto.getFromAccount())) {
                        result.add(chatRecordCondition);
                        copyList.remove(copyChatUserDto);
                        copyList.remove(chatRecordCondition);
                        break;
                    }
                }
            }
        }
        return CollectionUtils.isEmpty(result) ? 0 : result.size();
    }

}
